// Copyright Epic Games, Inc. All Rights Reserved.

#include "{{cookiecutter.project_name}}.h"
#include "Modules/ModuleManager.h"

IMPLEMENT_PRIMARY_GAME_MODULE( FDefaultGameModuleImpl, {{cookiecutter.project_name}}, "{{cookiecutter.project_name}}" );
